use qrcodegen::QrCode;
use qrcodegen::QrCodeEcc;
use std::fs::File;

use std::io::Write;
use std::path::PathBuf;

#[derive(Debug, Clone)]
pub struct QR {
    pub content: String,
}

impl QR {
    pub fn parse(content: &String) -> String {
        let qr = QrCode::encode_text(content, QrCodeEcc::Low).unwrap();
        QR::to_svg_string(&qr, 4)
    }
    // Returns a string of SVG code for an image depicting
    // the given QR Code, with the given number of border modules.
    // The string always uses Unix newlines (\n), regardless of the platform.
    pub fn to_svg_string(qr: &QrCode, border: i32) -> String {
        assert!(border >= 0, "Border must be non-negative");
        let mut result = String::new();
        result += "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        result += "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n";
        let dimension = qr
            .size()
            .checked_add(border.checked_mul(2).unwrap())
            .unwrap();
        result += &format!(
		"<svg xmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\" viewBox=\"0 0 {0} {0}\" stroke=\"none\">\n", dimension);
        result += "\t<rect width=\"100%\" height=\"100%\" fill=\"#FFFFFF\"/>\n";
        result += "\t<path d=\"";
        for y in 0..qr.size() {
            for x in 0..qr.size() {
                if qr.get_module(x, y) {
                    if x != 0 || y != 0 {
                        result += " ";
                    }
                    result += &format!("M{},{}h1v1h-1z", x + border, y + border);
                }
            }
        }
        result += "\" fill=\"#000000\"/>\n";
        result += "</svg>\n";
        result
    }

    pub fn print_qr(qr: &QrCode) {
        let border: i32 = 4;
        for y in -border..qr.size() + border {
            for x in -border..qr.size() + border {
                // 🀰 █
                let c: char = if qr.get_module(x, y) { '█' } else { ' ' };
                print!("{0}{0}", c);
            }
            println!();
        }
        println!();
    }

    pub fn write_image(data: &String, file_path: &PathBuf) {
        let file_name = "qr.svg";
        let mut path = file_path.clone();
        path.push(file_name);
        println!("{}", path.as_path().display());
        let mut file: File = match File::create(&path) {
            Err(why) => panic!("cannot create {}", why),
            Ok(f) => f,
        };
        match file.write_all(data.as_bytes()) {
            Ok(_) => println!("write success: {}", file_name),
            Err(why) => panic!("error write file: {}", why),
        }
    }
}
